@@ -131,6 +131,10 @@ public:
   /// @brief After \ref Calculate, estimated variance of coefficients (k x k)
   Matrix<Tv> BetaVar;
 
+  /// @brief Condition number which is 1-norm of variance beta multiplied by
+  /// 1-norm of its inverse
+  Tv condition_number = NAN;
+
   // @brief n x 1 (todo)
   // Matrix<Tv>* resid = nullptr;
 
@@ -323,6 +327,10 @@ public:
   /// \ref Projections)
   Tv CostRatioAvg = NAN;
 
+  /// @brief Brier score, the mean squared difference between predicted
+  /// probabilities and actual outcomes.
+  Tv BrierScore = NAN;
+
   /// @brief Initializes a new instance of this class
   /// @param modelType Type of model
   /// @param distType Type of distribution
@@ -410,7 +418,11 @@ public:
   Matrix<Tv> CostRatios;
 
   /// @brief After \ref Calculate, average value of AUC
-  Tv Auc = 0;
+  Tv Auc = NAN;
+
+  /// @brief Brier score, the mean squared difference between predicted
+  /// probabilities and actual outcomes.
+  Tv BrierScore = NAN;
 
   /// @brief Get a simulation class from types. Arguments are passed to the
   /// class or the constructors
@@ -418,8 +430,8 @@ public:
   GetFromType(bool hasWeight, DiscreteChoiceModelType modelType,
               DiscreteChoiceDistType distType, Ti rows, Ti cols, Ti numChoices,
               Tv trainPercentage, Ti trainFixSize, Ti costMatrixCount,
-              bool doAuc, bool doFrequecyTable, PcaAnalysisOptions *pcaOptions,
-              bool weightedEval);
+              bool doBrier, bool doAuc, bool doFrequecyTable,
+              PcaAnalysisOptions *pcaOptions, bool weightedEval);
 
   virtual void Calculate(const Matrix<Tv> &data,
                          const std::vector<Matrix<Tv>> *costMatrixes,
@@ -446,6 +458,7 @@ class LDT_EXPORT DiscreteChoiceSim : public DiscreteChoiceSimBase {
   Ti mNumChoices = 0;
   bool mDoAuc = false;
   bool mWeightedEval = false;
+  bool mDoBrier = false;
 
 public:
   /// @brief Initializes a new instance of this class
@@ -468,8 +481,8 @@ public:
   /// simulations
   /// @param weightedEval If true, weights are used in evaluations
   DiscreteChoiceSim(Ti rows, Ti cols, Ti numChoices, Tv trainPercentage,
-                    Ti trainFixSize, Ti costMatrixCount, bool doAuc,
-                    bool doFrequencyTable = false,
+                    Ti trainFixSize, Ti costMatrixCount, bool doBrier,
+                    bool doAuc, bool doFrequencyTable = false,
                     PcaAnalysisOptions *pcaOptions = nullptr,
                     bool weightedEval = false);
 
@@ -550,11 +563,10 @@ public:
   /// @brief Initializes a new instance of the class
   /// @param searchOptions It is passed to the base class
   /// @param searchItems It is passed to the base class
-  /// @param measures It is passed to the base class
+  /// @param metrics It is passed to the base class
   /// @param checks It is passed to the base class
   /// @param SizeG It is passed to the base class
   /// @param groupIndexMap It is passed to the base class
-  /// @param groupSizes It is passed to the base class
   /// @param fixFirstG It is passed to the base class
   /// @param source Data with variables in the columns
   /// @param numChoices Number of unique labels
@@ -563,13 +575,15 @@ public:
   /// the results
   /// @param newtonOptions Optimization options
   /// @param aucOptions Options for calculating AUC
-  DiscreteChoiceSearcher(
-      SearchOptions &searchOptions, const SearchItems &searchItems,
-      const SearchMeasureOptions &measures, const SearchModelChecks &checks,
-      Ti SizeG, const std::vector<std::vector<Ti>> &groupIndexMap,
-      const std::vector<Ti> &groupSizes, Ti fixFirstG, const Matrix<Tv> &source,
-      Ti numChoices, const std::vector<Matrix<Tv>> &costMatrixes,
-      unsigned int seed, Newton &newtonOptions, RocOptions &aucOptions);
+  DiscreteChoiceSearcher(SearchOptions &searchOptions,
+                         const SearchItems &searchItems,
+                         const SearchMetricOptions &metrics,
+                         const SearchModelChecks &checks, Ti SizeG,
+                         const std::vector<std::vector<Ti>> &groupIndexMap,
+                         Ti fixFirstG, const Matrix<Tv> &source, Ti numChoices,
+                         const std::vector<Matrix<Tv>> &costMatrixes,
+                         unsigned int seed, Newton &newtonOptions,
+                         RocOptions &aucOptions);
 };
 
 /// @brief A base class for a model set for discrete choice
@@ -598,9 +612,6 @@ public:
   /// be the owner
   std::vector<Searcher *> Searchers;
 
-  /// @brief Size of each member of \ref pGroupIndexMap
-  std::vector<Ti> GroupSizes;
-
   DiscreteChoiceModelsetBase(){};
 
   virtual ~DiscreteChoiceModelsetBase(){};
@@ -608,7 +619,7 @@ public:
   /// @brief Initializes the templated class from the types
   static DiscreteChoiceModelsetBase *
   GetFromTypes(bool isBinary, bool hasWeight, SearchOptions &searchOptions,
-               SearchItems &searchItems, SearchMeasureOptions &measures,
+               SearchItems &searchItems, SearchMetricOptions &metrics,
                SearchModelChecks &checks, const std::vector<Ti> &sizes,
                const Matrix<Tv> &source, std::vector<Matrix<Tv>> &costMatrixes,
                std::vector<std::vector<Ti>> &groupIndexMaps, bool addLogit,
@@ -631,7 +642,7 @@ public:
   /// @brief
   /// @param searchOptions It is passed to the base class
   /// @param searchItems It is passed to the base class
-  /// @param measures It is passed to the base class
+  /// @param metrics It is passed to the base class
   /// @param checks It is passed to the base class
   /// @param sizes Determines the number of exogenous data in different
   /// searchers
@@ -644,7 +655,7 @@ public:
   /// @param addLogit If true, logit models are included in the model set
   /// @param addProbit If true, probit models are included in the model set
   DiscreteChoiceModelset(SearchOptions &searchOptions, SearchItems &searchItems,
-                         SearchMeasureOptions &measures,
+                         SearchMetricOptions &metrics,
                          SearchModelChecks &checks,
                          const std::vector<Ti> &sizes, const Matrix<Tv> &source,
                          std::vector<Matrix<Tv>> &costMatrixes,
