@@ -5,7 +5,7 @@ using namespace Rcpp;
 using namespace ldt;
 
 void GetCostMatrices(bool printMsg, std::vector<ldt::Matrix<double>> &result,
-                     SEXP costMatrices, bool costMatInMeasures) {
+                     SEXP costMatrices, bool costMatInMetrics) {
   if (costMatrices != R_NilValue) {
     // don't use as.list or a vector such as c(c1,c2) will have invalid values
     if (is<List>(costMatrices) == false)
@@ -29,7 +29,7 @@ void GetCostMatrices(bool printMsg, std::vector<ldt::Matrix<double>> &result,
       Rprintf("    %i. Dimension=(%i,%i)\n", i + 1, result.at(i).RowsCount,
               result.at(i).ColsCount);
   }
-  if (costMatInMeasures && result.size() == 0)
+  if (costMatInMetrics && result.size() == 0)
     throw std::logic_error(
         "At least one frequency cost matrix is required for this type "
         "of out-of-sample evaluation.");
@@ -53,37 +53,11 @@ void checkData(ldt::Matrix<double> &my, ldt::Matrix<double> &mx,
     warning("Zero weight is found.");
 }
 
-// clang-format off
-
-//' Discrete Choice Search
-//'
-//' @param y (numeric matrix) endogenous data with variable in the column.
-//' @param x (numeric matrix) exogenous data with variables in the columns.
-//' @param w (numeric vector) weights of the observations in \code{y}. null means equal weights.
-//' @param xSizes (nullable int vector) Number of exogenous variables in the regressions. E.g., c(1,2) means the model set contains all the regressions with 1 and 2 exogenous variables. If null, c(1) is used.
-//' @param xPartitions (nullable list of int vector) a partition over the indexes of the exogenous variables. No regression is estimated with two variables in the same group. If null, each variable is placed in its own group and the size of the model set is maximized.
-//' @param costMatrices (list of numeric matrix) each frequency cost matrix determines how to score the calculated probabilities. Given the number of choices 'n', a frequency cost matrix is a 'm x n+1' matrix. The first column determines the thresholds. Cells in the j-th column determines the costs corresponding to the (j-1)-th choice in \code{y}. It can be null if it is not selected in \code{measureOptions}.
-//' @param searchLogit (bool) if \code{TRUE}, logit regressions are added to the model set.
-//' @param searchProbit (bool) if \code{TRUE}, probit regressions are added to the model set.
-//' @param optimOptions (nullable list) Newton optimization options. see \code{[GetNewtonOptions()]}.
-//' @param aucOptions (nullable list) AUC calculation options. see \code{[GetRocOptions()]}.
-//' @param measureOptions (nullable list) see \code{[GetMeasureOptions()]}.
-//' @param modelCheckItems (nullable list) see \code{[GetModelCheckItems()]}.
-//' @param searchItems (nullable list) see \code{[GetSearchItems()]}.
-//' @param searchOptions (nullable list) see \code{[GetSearchOptions()]}.
-//'
-//' @return A list
-//'
-//' @export
-// [[Rcpp::export]]
-SEXP DcSearch(SEXP y, SEXP x, SEXP w = R_NilValue, SEXP xSizes = R_NilValue,
-              SEXP xPartitions = R_NilValue, SEXP costMatrices = R_NilValue,
-              bool searchLogit = true, bool searchProbit = false,
-              SEXP optimOptions = R_NilValue, SEXP aucOptions = R_NilValue, SEXP measureOptions = R_NilValue,
-              SEXP modelCheckItems = R_NilValue, SEXP searchItems = R_NilValue,
-              SEXP searchOptions = R_NilValue)
-// clang-format on
-{
+// [[Rcpp::export(.SearchDc)]]
+SEXP SearchDc(SEXP y, SEXP x, SEXP w, SEXP xSizes, SEXP xPartitions,
+              SEXP costMatrices, bool searchLogit, bool searchProbit,
+              List optimOptions, List aucOptions, List metricOptions,
+              List modelCheckItems, List searchItems, List searchOptions) {
 
   if (y == R_NilValue || x == R_NilValue)
     throw std::logic_error("Invalid data: 'y' or 'x' is null.");
@@ -92,24 +66,12 @@ SEXP DcSearch(SEXP y, SEXP x, SEXP w = R_NilValue, SEXP xSizes = R_NilValue,
   if (is<NumericMatrix>(x) == false)
     throw std::logic_error("'x' must be a 'numeric matrix'.");
 
-  auto startTime = boost::posix_time::to_simple_string(
-      boost::posix_time::second_clock::local_time());
-
   int numTargets = 1;
 
-  List searchOptions_;
-  if (searchOptions != R_NilValue) {
-    if (is<List>(searchOptions) == false)
-      throw std::logic_error("'searchOptions' must be a 'List'.");
-    searchOptions_ = as<List>(searchOptions);
-    CheckSearchOptions(searchOptions_);
-  } else
-    searchOptions_ = GetSearchOptions();
-
   bool printMsg = false;
   auto options = SearchOptions();
   int reportInterval = 0;
-  UpdateSearchOptions(searchOptions_, options, reportInterval, printMsg);
+  UpdateSearchOptions(searchOptions, options, reportInterval, printMsg);
 
   y = as<NumericMatrix>(y);
   x = as<NumericMatrix>(x);
@@ -138,70 +100,22 @@ SEXP DcSearch(SEXP y, SEXP x, SEXP w = R_NilValue, SEXP xSizes = R_NilValue,
   GetPartitions(printMsg, xPartitions_, xPartitions, exoCount - 1, 0, true);
 
   Newton newton;
-  if (optimOptions == R_NilValue) {
-    List optimOptions_ = GetNewtonOptions();
-    UpdateNewtonOptions(printMsg, optimOptions_, newton);
-  } else {
-    if (is<List>(optimOptions) == false)
-      throw std::logic_error("'optimOptions' must be a 'List'.");
-    List optimOptions_ = as<List>(optimOptions);
-    CheckNewtonOptions(optimOptions_);
-    UpdateNewtonOptions(printMsg, optimOptions_, newton);
-  }
+  UpdateNewtonOptions(printMsg, optimOptions, newton);
 
   RocOptions aucOptions0;
-  if (aucOptions == R_NilValue) {
-    List aucOptions_ = GetRocOptions();
-    UpdateRocOptions(printMsg, aucOptions_, aucOptions0, "AUC Options:");
-  } else {
-    if (is<List>(aucOptions) == false)
-      throw std::logic_error("'aucOption' must be a 'List'.");
-    auto aucOptions_ = as<List>(aucOptions);
-    CheckRocOptions(aucOptions_);
-    UpdateRocOptions(printMsg, aucOptions_, aucOptions0, "AUC Options:");
-  }
+  UpdateRocOptions(printMsg, aucOptions, aucOptions0, "AUC Options:");
 
-  List measureOptions_;
-  if (measureOptions == R_NilValue)
-    measureOptions_ = GetMeasureOptions();
-  else {
-    if (is<List>(measureOptions) == false)
-      throw std::logic_error("'measureOptions' must be a 'List'.");
-    measureOptions_ = as<List>(measureOptions);
-    CheckMeasureOptions(measureOptions_);
-  }
-
-  List modelCheckItems_;
-  if (modelCheckItems == R_NilValue)
-    modelCheckItems_ = GetModelCheckItems();
-  else {
-    if (is<List>(modelCheckItems) == false)
-      throw std::logic_error("'modelCheckItems' must be a 'List'.");
-    modelCheckItems_ = as<List>(modelCheckItems);
-    CheckModelCheckItems(modelCheckItems_);
-  }
-
-  List searchItems_;
-  if (searchItems == R_NilValue)
-    searchItems_ = GetSearchItems();
-  else {
-    if (is<List>(searchItems) == false)
-      throw std::logic_error("'searchItems' must be a 'List'.");
-    searchItems_ = as<List>(searchItems);
-    CheckSearchItems(searchItems_);
-  }
-
-  auto measures = SearchMeasureOptions();
-  auto measuresNames = std::vector<std::string>();
+  auto metrics = SearchMetricOptions();
+  auto metricsNames = std::vector<std::string>();
   auto items = SearchItems();
   auto checks = SearchModelChecks();
-  UpdateOptions(printMsg, searchItems_, measureOptions_, modelCheckItems_,
-                measures, items, checks, measuresNames, exoCount, exoCount,
-                numTargets, 1, false, true, "Coefficients", true);
+  UpdateOptions(printMsg, searchItems, metricOptions, modelCheckItems, metrics,
+                items, checks, metricsNames, exoCount, exoCount, numTargets, 1,
+                false, true, "Coefficients", true);
 
   std::vector<ldt::Matrix<double>> costMatrices0;
   GetCostMatrices(printMsg, costMatrices0, costMatrices,
-                  Contains(measures.MeasuresOut, ScoringType::kFrequencyCost));
+                  Contains(metrics.MetricsOut, ScoringType::kFrequencyCost));
 
   // get parameter names
   std::vector<std::string> paramNames;
@@ -219,7 +133,7 @@ SEXP DcSearch(SEXP y, SEXP x, SEXP w = R_NilValue, SEXP xSizes = R_NilValue,
                                           : (searchLogit ? "Logit" : "Probit"));
 
   DiscreteChoiceModelsetBase *model = DiscreteChoiceModelsetBase::GetFromTypes(
-      isBinary, hasWeight, options, items, measures, checks, xSizes_, mat,
+      isBinary, hasWeight, options, items, metrics, checks, xSizes_, mat,
       costMatrices0, xPartitions_, searchLogit, searchProbit, newton,
       aucOptions0);
   auto modelr = std::unique_ptr<DiscreteChoiceModelsetBase>(model);
@@ -240,6 +154,8 @@ SEXP DcSearch(SEXP y, SEXP x, SEXP w = R_NilValue, SEXP xSizes = R_NilValue,
     throw std::logic_error("More memory is required for running the project.");
   }
 
+  auto alli = model->Modelset.GetExpectedNumberOfModels();
+
   // handle unhandled exceptions in the async function
   // model->CheckStart();
   auto f = std::async(std::launch::async, [&] {
@@ -247,11 +163,8 @@ SEXP DcSearch(SEXP y, SEXP x, SEXP w = R_NilValue, SEXP xSizes = R_NilValue,
     estimating = false;
   });
 
-  ReportProgress(printMsg, reportInterval, model->Modelset, estimating,
-                 options);
-
-  auto endTime = boost::posix_time::to_simple_string(
-      boost::posix_time::second_clock::local_time());
+  ReportProgress(printMsg, reportInterval, model->Modelset, estimating, options,
+                 alli);
 
   if (options.RequestCancel)
     return R_NilValue;
@@ -262,56 +175,30 @@ SEXP DcSearch(SEXP y, SEXP x, SEXP w = R_NilValue, SEXP xSizes = R_NilValue,
 
   auto extraLabel = "dist";
   auto extraNames = std::vector<std::string>({"dist"});
-  List L = GetModelSetResults(model->Modelset, items, measuresNames, exoCount,
+  List L = GetModelSetResults(model->Modelset, items, metricsNames, exoCount,
                               extraLabel, &extraNames, 1, paramNames,
                               colNames_w, "coefs", "item");
 
   L["info"] = List::create(
-      _["startTime"] = wrap(startTime), _["endTime"] = wrap(endTime),
       _["yNames"] = colnames(y), _["xNames"] = colnames(x),
       _["costMatrices"] = costMatrices, _["searchLogit"] = wrap(searchLogit),
       _["searchProbit"] = wrap(searchProbit), _["optimOptions"] = optimOptions,
-      _["measureOptions"] = measureOptions_,
-      _["modelCheckItems"] = modelCheckItems_, _["searchItems"] = searchItems_,
-      _["searchOptions"] = searchOptions_, _["numTargets"] = wrap(numTargets));
+      _["metricOptions"] = metricOptions,
+      _["modelCheckItems"] = modelCheckItems, _["searchItems"] = searchItems,
+      _["searchOptions"] = searchOptions, _["numTargets"] = wrap(numTargets));
 
   L.attr("class") =
       std::vector<std::string>({"ldtsearchdc", "ldtsearch", "list"});
-  L.attr("method") = "dc";
+  L.attr("method") = "bin";
 
   return L;
 }
 
-// clang-format off
-
-//' Estimates an Discrete Choice Model
-//'
-//' @param y (numeric matrix) Data with dependent variable in the column. Given the number of choices 'n', it must contain 0,1,...,n-1 and 'sum(y==i)>0' for i=0,...,n-1.
-//' @param x (numeric matrix) Exogenous data with variables in the columns.
-//' @param w (numeric vector) Weights of the observations in \code{y}. Null means equal weights.
-//' @param distType (string) Distribution assumption. It can be \code{logit} or \code{probit}.
-//' @param newX (numeric matrix) If not null, probabilities are projected for each row of this matrix.
-//' @param pcaOptionsX (list) A list of options in order to use principal components of the \code{x}, instead of the actual values. set null to disable. Use [GetPcaOptions()] for initialization.
-//' @param costMatrices (list of matrices) Each cost table determines how you score the calculated probabilities.
-//' @param aucOptions (nullable list) AUC calculation options. see \code{[GetRocOptions()]}.
-//' @param simFixSize (int) Number of pseudo out-of-sample simulations. Use zero to disable the simulation. (see [GetMeasureOptions()]).
-//' @param simTrainRatio (double) Size of the training sample as a ratio of the number of the observations. It is effective only if \code{simTrainFixSize} is zero.
-//' @param simTrainFixSize (int) A fixed size for the training sample. If zero, \code{simTrainRatio} is used.
-//' @param simSeed (int) A seed for the pseudo out-of-sample simulation.
-//' @param weightedEval (bool) If true, weights will be used in evaluations.
-//' @param printMsg (bool) Set false to disable printing the details.
-//'
-//' @return A list:
-//'
-//' @export
-// [[Rcpp::export]]
-SEXP DcEstim(SEXP y, SEXP x, SEXP w = R_NilValue,
-             std::string distType = "logit", SEXP newX = R_NilValue,
-             SEXP pcaOptionsX = R_NilValue, SEXP costMatrices = R_NilValue,
-             SEXP aucOptions = R_NilValue, int simFixSize = 200, double simTrainRatio = 0.5,
-             int simTrainFixSize = 0, int simSeed = 0, bool weightedEval = false, bool printMsg = false)
-// clang-format on
-{
+// [[Rcpp::export(.EstimDc)]]
+SEXP EstimDc(SEXP y, SEXP x, SEXP w, std::string linkFunc, SEXP newX,
+             SEXP pcaOptionsX, SEXP costMatrices, List aucOptions,
+             int simFixSize, double simTrainRatio, int simTrainFixSize,
+             int simSeed, bool weightedEval, bool printMsg) {
   if (y == R_NilValue || x == R_NilValue)
     throw std::logic_error("Invalid data: 'y' or 'x' is null.");
   if (is<NumericMatrix>(y) == false)
@@ -322,7 +209,7 @@ SEXP DcEstim(SEXP y, SEXP x, SEXP w = R_NilValue,
   y = as<NumericMatrix>(y);
   x = as<NumericMatrix>(x);
 
-  if (newX != R_NilValue){
+  if (newX != R_NilValue) {
     if (is<NumericMatrix>(newX) == false)
       throw std::logic_error("'newX' must be a 'numeric matrix'.");
     newX = insert_intercept(
@@ -380,7 +267,7 @@ SEXP DcEstim(SEXP y, SEXP x, SEXP w = R_NilValue,
   DiscreteChoiceModelType modelType0 =
       FromString_DiscreteChoiceModelType(modelType);
   DiscreteChoiceDistType distType0 =
-      FromString_DiscreteChoiceDistType(distType.c_str());
+      FromString_DiscreteChoiceDistType(linkFunc.c_str());
   if (printMsg) {
     Rprintf("Model Type=%s\n", ToString(modelType0));
     Rprintf("Distribution Type=%s\n", ToString(distType0));
@@ -391,16 +278,7 @@ SEXP DcEstim(SEXP y, SEXP x, SEXP w = R_NilValue,
     Rprintf("Number of Projections=%i\n", mnewX.RowsCount);
 
   RocOptions aucOptions0;
-  if (aucOptions == R_NilValue) {
-    List aucOptions_ = GetRocOptions();
-    UpdateRocOptions(printMsg, aucOptions_, aucOptions0, "AUC Options:");
-  } else {
-    if (is<List>(aucOptions) == false)
-      throw std::logic_error("Invalid 'aucOption'. It is not a list.");
-    auto aucOptions_ = as<List>(aucOptions);
-    CheckRocOptions(aucOptions_);
-    UpdateRocOptions(printMsg, aucOptions_, aucOptions0, "AUC Options:");
-  }
+  UpdateRocOptions(printMsg, aucOptions, aucOptions0, "AUC Options:");
 
   // ESTIMATE
 
@@ -434,7 +312,7 @@ SEXP DcEstim(SEXP y, SEXP x, SEXP w = R_NilValue,
     simmodel = DiscreteChoiceSimBase::GetFromType(
         hasWeight, modelType0, distType0, mat.RowsCount, mat.ColsCount,
         numChoices, simTrainRatio, simTrainFixSize, costMatrices0.size(), true,
-        true, hasPcaX ? &pcaOptions0 : nullptr, weightedEval);
+        true, true, hasPcaX ? &pcaOptions0 : nullptr, weightedEval);
     simmodelf = std::unique_ptr<DiscreteChoiceSimBase>(simmodel);
     simmodel->SimulationMax = simFixSize;
     simmodel->Seed = simSeed;
@@ -480,29 +358,32 @@ SEXP DcEstim(SEXP y, SEXP x, SEXP w = R_NilValue,
   } else
     exoNames_pca = exoNames;
 
-  // Measures
-  int measureCount = 5; // logL, aic, sic, aucIn, costIn,
+  // Metrics
+  int metricCount = 6; // logL, aic, sic, brierIn aucIn, costIn,
   if (simFixSize > 0)
-    measureCount += 2; // aucOut, costOut
+    metricCount += 3; // brierOur aucOut, costOut
   int eqCount = 1;
-  auto measuresResD = std::unique_ptr<double[]>(
-      new double[measureCount * eqCount]); // for 1 equation
-  auto measuresRes =
-      ldt::Matrix<double>(measuresResD.get(), measureCount, eqCount);
-  auto measuresResRowNames = std::vector<std::string>(
-      {"logL", "aic", "sic",
-       "aucIn", // keep names consistent with the measures in the search
+  auto metricsResD = std::unique_ptr<double[]>(
+      new double[metricCount * eqCount]); // for 1 equation
+  auto metricsRes =
+      ldt::Matrix<double>(metricsResD.get(), metricCount, eqCount);
+  auto metricsResRowNames = std::vector<std::string>(
+      {"logL", "aic", "sic", "brierIn",
+       "aucIn", // keep names consistent with the metrics in the estimation
        "frequencyCostIn"});
-  measuresRes.Set(0, 0, model.Model->LogL);
-  measuresRes.Set(1, 0, model.Model->Aic);
-  measuresRes.Set(2, 0, model.Model->Sic);
-  measuresRes.Set(3, 0, model.Auc);
-  measuresRes.Set(4, 0, model.CostRatioAvg);
+  metricsRes.Set0(0, 0, model.Model->LogL);
+  metricsRes.Set0(1, 0, model.Model->Aic);
+  metricsRes.Set0(2, 0, model.Model->Sic);
+  metricsRes.Set0(3, 0, model.BrierScore);
+  metricsRes.Set0(4, 0, model.Auc);
+  metricsRes.Set0(5, 0, model.CostRatioAvg);
   if (simFixSize > 0) {
-    measuresResRowNames.push_back("aucOut");
-    measuresResRowNames.push_back("frequencyCostOut");
-    measuresRes.Set(5, 0, simmodel->Auc);
-    measuresRes.Set(6, 0, simmodel->CostRatios.Mean());
+    metricsResRowNames.push_back("brierOut");
+    metricsResRowNames.push_back("aucOut");
+    metricsResRowNames.push_back("frequencyCostOut");
+    metricsRes.Set0(6, 0, simmodel->BrierScore);
+    metricsRes.Set0(7, 0, simmodel->Auc);
+    metricsRes.Set0(8, 0, simmodel->CostRatios.Mean());
   }
 
   List L = List::create(
@@ -520,7 +401,7 @@ SEXP DcEstim(SEXP y, SEXP x, SEXP w = R_NilValue,
               as_matrix(model.Model->BetaProb, &exoNames_pca, &endoNames),
           _["gamma"] = as_matrix(model.Model->Beta),
           _["gammaVar"] = as_matrix(model.Model->BetaVar)),
-      _["measures"] = as_matrix(measuresRes, &measuresResRowNames, &endoNames),
+      _["metrics"] = as_matrix(metricsRes, &metricsResRowNames, &endoNames),
       _["projection"] = hasProjection == false
                             ? R_NilValue
                             : (SEXP)as_matrix(model.PredProbs),
@@ -540,7 +421,7 @@ SEXP DcEstim(SEXP y, SEXP x, SEXP w = R_NilValue,
 
   L.attr("class") =
       std::vector<std::string>({"ldtestimdc", "ldtestim", "list"});
-  L.attr("method") = "dc";
+  L.attr("method") = "bin";
 
   return L;
 }
