@@ -69,25 +69,17 @@ static Tv F(Tv r2, Ti N, Ti m, Ti qstar, Tv &pvalue) {
 
 static double logL(Matrix<Tv> &resid_var_copy, Ti N, Ti m) {
   // see Greene p. 348
-  auto W = Matrix<Tv>(new double[m * m], m, m);
+  auto Md = std::unique_ptr<Tv[]>(new Tv[m * m]);
+  auto W = Matrix<Tv>(Md.get(), m, m);
   resid_var_copy.CopyTo00(W);
-  Tv resid_var_det = W.Det_pd0();
-  resid_var_copy.CopyTo00(W);
-  W.Multiply_in((Tv)N);
-  // resid.Dot_AtA(W);
+  Tv resid_var_det = W.Det_pd0(); // |W|
 
   if (std::isnan(resid_var_det))
     throw std::logic_error("Determinant of residual variance is NAN");
 
-  auto SW = Matrix<Tv>(new double[m * m], m, m);
-  resid_var_copy.Inv0();
-  resid_var_copy.Dot(W, SW);
+  auto ll =
+      (Tv)-0.5 * N * (m * c_ln_2Pi + std::log(resid_var_det)) - (Tv)0.5 * m * N;
 
-  auto ll = (Tv)-0.5 * N * (m * c_ln_2Pi + std::log(resid_var_det)) -
-            (Tv)0.5 * SW.Trace();
-
-  delete[] W.Data;
-  delete[] SW.Data;
   return ll;
   /*
   Tv _n_2 = -(N / (Tv)2);
@@ -97,14 +89,12 @@ static double logL(Matrix<Tv> &resid_var_copy, Ti N, Ti m) {
   */
 }
 
-static Tv aic(Tv logL, Ti N, Ti k) { return (2 * k - 2 * logL) / (Tv)N; }
+static Tv aic(Tv logL, Ti N, Ti k) { return (2 * k - 2 * logL); }
 
-static Tv sic(Tv logL, Ti N, Ti k) {
-  return (std::log(N) * k - 2 * logL) / (Tv)N;
-}
+static Tv sic(Tv logL, Ti N, Ti k) { return (std::log(N) * k - 2 * logL); }
 
 static Tv hqic(Tv logL, Ti N, Ti k) {
-  return (std::log(std::log(N)) * 2 * k - 2 * logL) / (Tv)N;
+  return (std::log(std::log(N)) * 2 * k - 2 * logL);
 }
 
 // static Tv aic_weight(Tv criterion) { return std::exp(((Tv)-0.5) * criterion);
@@ -149,10 +139,10 @@ void Sur::calculate_details(Ti N, Ti m, Tv *work, bool just_probs,
 
   // Goodness of Fit
   r2 = R2(y, resid);
-  f = F(r2, N, m, k0 * m, f_prob);
-  Aic = aic(logLikelihood, N, m);
-  Sic = sic(logLikelihood, N, m);
-  Hqic = hqic(logLikelihood, N, m);
+  f = F(r2, N, m, k0m, f_prob);
+  Aic = aic(logLikelihood, N, k0m);
+  Sic = sic(logLikelihood, N, k0m);
+  Hqic = hqic(logLikelihood, N, k0m);
 }
 
 void Sur::estim_un(Ti N, Ti m, Tv *work,
@@ -180,7 +170,7 @@ void Sur::estim_un(Ti N, Ti m, Tv *work,
   condition_number = xtx.Norm('1'); // condition number
   auto info = xtx.Inv0();
   if (info != 0) {
-    throw exp_mat_sin; // Matrix<Tv> singularity
+    throw std::logic_error("matrix singularity");
     return;
   }
   condition_number *= xtx.Norm('1');
@@ -253,17 +243,16 @@ void Sur::estim_r(Ti N, Ti m, Tv *work) {
   auto V_o_xRtz = Matrix<Tv>(&work[q], qStar, (Ti)1);
   q += qStar;
 
-  resid_var.Inv0();
-  x.Dot_AtA(xtx); // kxk
-  resid_var.Kron(xtx, V_o_xtx);
-  pR->TrDot(V_o_xtx, RV_o_xtx);
-  RV_o_xtx.Dot(*pR, gamma_var);
+  resid_var.Inv0();             // S^-1
+  x.Dot_AtA(xtx);               // X'X: kxk
+  resid_var.Kron(xtx, V_o_xtx); // S^-1 o X'X
+  pR->TrDot(V_o_xtx, RV_o_xtx); // R'[S^-1 o X'X]
+  RV_o_xtx.Dot(*pR, gamma_var); // R'[S^-1 o X'X]R
 
   condition_number = RV_o_xtx.Norm('1');
-  auto info =
-      gamma_var.Inv0(); //                      [R'(V^{-1} o pX'pX)R]^{-1}
+  auto info = gamma_var.Inv0(); // [R'[S^-1 o X'X]R]^{-1}
   if (info != 0) {
-    throw exp_mat_sin; // Matrix<Tv> singularity
+    throw std::logic_error("matrix singularity");
     return;
   }
   condition_number *= gamma_var.Norm('1');
@@ -275,8 +264,8 @@ void Sur::estim_r(Ti N, Ti m, Tv *work) {
   //     of course the estimator of resid_var can be different. It can be
   //     Identity, a diagonal Matrix, ...
 
-  resid_var.Kron(x, V_o_x);
-  V_o_x.Dot(*pR, V_o_xR);
+  resid_var.Kron(x, V_o_x); // S^-1 o x
+  V_o_x.Dot(*pR, V_o_xR);   // [S^-1 o x]R
   if (pr) {
     throw std::logic_error("not implemented (with r restriction)");
     // this is wrong. Kronecker is with I
@@ -285,22 +274,22 @@ void Sur::estim_r(Ti N, Ti m, Tv *work) {
     y.Subtract0(I_o_xr, z);     // subtract0 for vec(y) ??? see 'else'
     V_o_xR.TrDot0(z, V_o_xRtz); // dot0 for vec(y) ??? see 'else'
   } else {
-    y.Restructure0(Nm, 1); // to vec(y)
-    V_o_xR.TrDot(y, V_o_xRtz);
+    y.Restructure0(Nm, 1);     // to vec(y)
+    V_o_xR.TrDot(y, V_o_xRtz); // R'[S^-1 o x']vec(y)
     y.Restructure0(N, m);
   }
 
   gamma.Restructure0(qStar, (Ti)1);
-  gamma_var.Dot(V_o_xRtz, gamma);
+  gamma_var.Dot(V_o_xRtz, gamma); // [R'[S^-1 o X'X]R]^{-1} R'[S^-1 o x']vec(y)
 
   // convert gamma to beta
-  pR->Dot0(gamma, beta);
+  pR->Dot0(gamma, beta); // B = R[R'[S^-1 o X'X]R]^{-1} R'[S^-1 o x']vec(y)
   if (pr) {
     beta.Add_in0(*pr);
   }
 
   // same as before:
-  x.Dot(beta, yhat);
+  x.Dot(beta, yhat); // Y^ = x B
   y.Subtract(yhat, resid);
   resid.Dot_AtA(resid_var);
   resid_var.Divide_in((
@@ -347,7 +336,7 @@ void Sur::estim_search(Ti N, Ti m, Tv *work, Tv sigSearchMaxProb) {
     pR->SetValue(0);
     j = 0;
     for (auto i : sig_inds) {
-      pR->Set(i, j, 1); // rows with significant coefficient must have 1
+      pR->Set0(i, j, 1); // rows with significant coefficient must have 1
       j++;
     }
 
